from django.db import models
from django.utils.translation import ugettext, ugettext_lazy as _
from django.template.defaultfilters import slugify
from django.core.urlresolvers import reverse
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.models import User

MONTHS = (
	(1, _('January')),
	(2, _('February')),
	(3, _('March')),
	(4, _('April')),
	(5, _('May')),
	(6, _('June')),
	(7, _('July')),
	(8, _('August')),
	(9, _('September')),
	(10, _('October')),
	(11, _('November')),
	(12, _('December'))
)

class Publication(models.Model):
	resource	= models.FileField(upload_to = 'uploads/publications', null = True, blank = True)
	title 		= models.CharField(max_length = 255)
	year 		= models.IntegerField(help_text = _('The year must be greater than 0 and lesser than 9999'))
	member_authors = models.ManyToManyField(User)
	other_authors = models.TextField(help_text = _('Separate the authors using "and". Do not use commas.'), blank=True, null=True)
	real_type = models.ForeignKey(ContentType, editable=False, null=True)

	class Meta:
		ordering = ['title']

	def __unicode__(self):
		return self.title

	def save(self, *args, **kwargs):
		if not self.id:
			self.real_type = self._get_real_type()
		super(Publication, self).save(*args, **kwargs)

	def _get_real_type(self):
		return ContentType.objects.get_for_model(type(self))

	def cast(self):
		return self.real_type.get_object_for_this_type(pk=self.pk)
		
	def month_to_bibtex(self, month):
		str_out = ''
		if (month == 1):
			str_out = 'jan'
		elif (month == 2):
			str_out = 'feb'
		elif (month == 3):
			str_out = 'mar'
		elif (month == 4):
			str_out = 'apr'
		elif (month == 5):
			str_out = 'may'
		elif (month == 6):
			str_out = 'jun'
		elif (month == 7):
			str_out = 'jul'
		elif (month == 8):
			str_out = 'aug'
		elif (month == 9):
			str_out = 'sep'
		elif (month == 10):
			str_out = 'oct'
		elif (month == 11):
			str_out = 'nov'
		else:
			str_out = 'dec'
		return str_out
		
	def get_resource_as_link(self):
		if self.resource:
			return "<a href=\"%s\">%s</a>" % (self.resource.url, _("Resource"))
		else:
			return None
	get_resource_as_link.allow_tags = True
	
	def to_bibtex_as_link(self):
		return "<a href=\"%s\" target=\"_blank\">%s</a>" % (reverse("download_bibtex", 
										  args=[self.id]), "Download")	
	to_bibtex_as_link.allow_tags = True	
	
	def get_authors_names(self):
		#self.member_authors
		pass
				
class JournalArticle(Publication):
	journal		= models.CharField(max_length = 255)
	volume		= models.IntegerField(blank = True)
	number		= models.IntegerField(blank = True)
	pages		= models.CharField(max_length = 20, blank = True)
	
	class Meta:
		verbose_name = "Journal Article"
		
	def to_bibtex(self):
		str_out = ['@article{', slugify(self.title), ', ', 
			'author = "',  '", ',
			'title = "', self.title, '", ',
			'year = ', str(self.year), ', ',
			'journal = "', self.journal, '", ',
			'volume = ', str(self.volume), ', ',
			'number = ', str(self.number), ', ',
			'pages = "', self.pages, '"}']
		return ''.join(str_out)
		
class ConferenceArticle(Publication):
	conference	= models.CharField(max_length = 255)
	pages		= models.CharField(max_length = 20, blank = True)
	month 		= models.IntegerField(blank = True, choices = MONTHS)
	
	class Meta:
		verbose_name = "Conference Article"
		
	def to_bibtex(self):
		str_out = ['@conference{', slugify(self.title), ', ', 
			'author = "',  '", ',
			'title = "', self.title, '", ',
			'year = ', str(self.year), ', ',
			'conference = "', self.conference, '", ',
			'month = ', self.month_to_bibtex(self.month), ', ',
			'pages = "', self.pages, '"}']
		return ''.join(str_out)

class MScDissertation(Publication):
	school		= models.CharField(max_length = 255)
	month 		= models.IntegerField(blank = True, choices = MONTHS)
	
	class Meta:
		verbose_name = "MSc Dissertation"
		
	def to_bibtex(self):
		str_out = ['@mastersthesis{', slugify(self.title), ', ', 
			'author = "',  '", ',
			'title = "', self.title, '", ',
			'year = ', str(self.year), ', ',
			'school = "', self.school, '", ',
			'month = ', self.month_to_bibtex(self.month), '"}']
		return ''.join(str_out)
	
class PhDThesis(Publication):
	school		= models.CharField(max_length = 255)
	month 		= models.IntegerField(blank = True, choices = MONTHS)
	
	class Meta:
		verbose_name = "PhD Thesis"
		verbose_name_plural = "PhD Theses"
		
	def to_bibtex(self):
		str_out = ['@phdthesis{', slugify(self.title), ', ', 
			'author = "',  '", ',
			'title = "', self.title, '", ',
			'year = ', str(self.year), ', ',
			'school = "', self.school, '", ',
			'month = ', self.month_to_bibtex(self.month), '"}']
		return ''.join(str_out)
